@@ -18,6 +18,7 @@
 import asyncio
 import time
 import traceback
+import psutil
 
 import websockets, requests, urllib, json
 import os, shutil, pathlib
@@ -42,7 +43,10 @@ class Bot:
 
     def __init__(self, token, sequence = 0, session_id = None, bot_identifier = "python/discordbot/v1.0"):
         try:
-            assert not "active" in os.listdir(file_root)
+            if "active" in os.listdir(file_root):
+                with open(file_root / "active", "r") as f:
+                    i = int(f.read())
+                    assert not psutil.pid_exists(i)
         except:
             raise error.BotAlreadyRunning("The bot appears to already be running. If the bot is not running, then please wait up to 30 seconds for the previous bot to shut down. After that, make sure that %s exists and is EMPTY (you may delete any present files)." % file_root)
         
@@ -79,6 +83,7 @@ class Bot:
         async def logout(dt):
             if self.stopped:
                 await self.gateway_connection.close(1001)
+                self.gateway_connection = None
         self.register_periodic(0.1, logout)
         
         # register command handler
@@ -312,7 +317,7 @@ class Bot:
             self.log("Unknown payload received: %s" % pl, severity = log_severity.warning)
     
     async def _heartbeat(self, dt):
-        if not self.ready:
+        if not self.gateway_connection:
             return
         await self._send_payload(api.payload.OutgoingHeartbeat(self.last_received_sequence))
     
@@ -331,6 +336,7 @@ class Bot:
             self.gateway_connection = await websockets.connect(self.gateway_endpoint)
         except BaseException as e:
             self.log_error(e)
+            self.gateway_connection = None
             return 2, "Could not connect to gateway"
         self.log("found gateway: %s" % self.gateway_endpoint)
         
@@ -341,6 +347,7 @@ class Bot:
         except BaseException as e:
             self.log_error(e)
             await self.gateway_connection.close(1000) # will not reconnect
+            self.gateway_connection = None
             return 3, "Failed to receive hello payload"
         self.log("received hello")
         
@@ -356,6 +363,7 @@ class Bot:
         except BaseException as e:
             self.log_error(e)
             await self.gateway_connection.close(1000) # will not reconnect
+            self.gateway_connection = None
             return 4, "Failed to send initial heartbeat"
         self.log("sent initial heartbeat")
             
@@ -365,6 +373,7 @@ class Bot:
         except BaseException as e:
             self.log_error(e)
             await self.gateway_connection.close(1000) # will not reconnect
+            self.gateway_connection = None
             return 5, "Failed to receive initial heartbeat ACK"
         self.log("received initial heartbeat ACK")
         
@@ -381,6 +390,7 @@ class Bot:
             except BaseException as e:
                 self.log_error(e)
                 await self.gateway_connection.close(1000) # will not reconnect
+                self.gateway_connection = None
                 return 6, "Failed to resume"
             self.log("resumed using: %s" % r.dumps())
         else:
@@ -394,6 +404,7 @@ class Bot:
             except BaseException as e:
                 self.log_error(e)
                 await self.gateway_connection.close(1000) # will not reconnect
+                self.gateway_connection = None
                 return 7, "Failed to identify"
             self.log("identified")
             
@@ -404,6 +415,7 @@ class Bot:
             except BaseException as e:
                 self.log_error(e)
                 await self.gateway_connection.close(1000) # will not reconnect
+                self.gateway_connection = None
                 return 8, "Failed to receive ready event"
             self.log("received ready event. session_id: %s" % ready.session_id)
             self.user = ready.user
@@ -432,6 +444,7 @@ class Bot:
                         await self.gateway_connection.close(1012) # will reconnect
                     except:
                         pass
+                    self.gateway_connection = None
                     return await self._run()
         except (KeyboardInterrupt, asyncio.exceptions.CancelledError) as e:
             self.stopped = True
@@ -439,9 +452,9 @@ class Bot:
                 await self.gateway_connection.close(1000) # will not reconnect
             except:
                 pass
+            self.gateway_connection = None
             return 0, "KeyboardInterrupt"
         except (BaseException, asyncio.IncompleteReadError) as e:
-            
             if self.stopped:
                 try:
                     await self.gateway_connection.close(1000) # will not reconnect
@@ -453,6 +466,7 @@ class Bot:
                     await self.gateway_connection.close(1012) # will reconnect
                 except:
                     pass
+            self.gateway_connection = None
             
             if isinstance(e, asyncio.IncompleteReadError):
                 self.log("Reconnecting due to an IncompleteReadError: %s" % e)
